#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
#   abaqusinp2fistrcnt.py
#
#       abaqus（inpファイル）の境界条件をfistr（cntファイル）に変換
#
#   16/01/12    新規作成
#      01/16    完成
#      02/04    inpファイル読み込み時大文字に変換してファイル変換
#               大文字、小文字関係なくファイル変換可能
#      02/05    小文字のgroup名を残す様に修正
#   18/11/30    convNoToName:境界条件が直接節点No、要素Noで設定されている場合、
#               その数が10以上有る場合は、設定しないように修正。
#   19/05/08    python3用に書き換え
#

import os
import sys

#  変換内容
def convCont():
    #         abaqus                 fistr
    cont = [["BOUNDARY",          "BOUNDARY"],          #FIXTEMPを含む
            ["CLOAD",             "CLOAD"],
            ["DLOAD",             "DLOAD"],
            ["SPRING",            "SPRING"],
            ["INITIAL CONDITIONS","INITIAL CONDITION"],
            ["TEMPERATURE",       "TEMPERATURE"],
            ["CFLUX",             "CFLUX"]]
    return cont

#  項目数
def nItemsInBC():
    #          BC名     組数 default値  対象
    cont = [["BOUNDARY", 3,  0.0,       "node"],
            ["CLOAD",    3,  0.0,       "node"],
            ["DLOAD",    1,  0.0,       "element"],
            ["SPRING",   3,  0.0,       "node"],
            ["INITIAL CONDITION", 1, 0.0, "node"],
            ["TEMPERATURE", 1, 0.0,     "node"],
            ["CFLUX",    1,  0.0,       "node"]]
    return cont

#  printMsg
#    print help
def printMsg():
    msg = """
<abaqusinp2fistrcnt.py>
abaqusのinpファイルから境界条件を読み取り、FrontISTRのcntファイルに書き込む。

<使い方>
abaqusinp2fistrcnt.py <inpファイル>

例：
abaqusinp2fistrcnt.py name.inp
abaqusinp2fistrcnt.py name
"""
    print (msg)

#
#  isConvLine
#    変換対象のlineを取得
def isConvLine(line):
    ans = False
    word = line.split(",")[0][1:].upper()
    commWord = " ".join(word.split())
    comms = convCont()
    for i in range(len(comms)):
        comm = comms[i]
        if commWord == comm[0]:
            ans = True
            break
    return ans

#  deleteSp
#    空白、CR,LFを削除
def deleteSp(line):
    newLine = ""
    for chara in line:
        if chara != " " and chara != "\n" and chara != "\r":
            newLine += chara
    return newLine

#
#  getBoundaryConditions
#    境界条件部を取得
def getBoundaryConditions(lines):
    getLines = []
    flag = 0
    for n in range(len(lines)):
        line = lines[n]
        if line[0] == "*":
            flag = 0
            if isConvLine(line) == True:
                if n+1 < len(lines) and lines[n+1][0] != "*":
                    #header部を取得
                    getLines.append(line)
                    flag = 1
        else:
            if flag == 1:
                #data部を取得
                getLines.append(line)
    #取得したlinesをprint
    i = 0
    while i < len(getLines):
        [bcLines, i] = get1BcLines(getLines, i)
        if len(bcLines) > 0:
            header = bcLines[0]
            dataLines = bcLines[1:]
            headerName = " ".join((header.split(",")[0]).split())
            print (" found " + headerName + " ", end=" ")
            for line in dataLines[:5]:
                print (deleteSp(line.split(",")[0]), end=" ")
            if len(dataLines) > 5:
                print ("...", end=" ")
            print
    return getLines

#
#  checkGrpNameInHeader
#    header内のgrpNameをチェック
#     header内には1ヶのgrpName
def checkGrpNameInHeader(lines):
    newLines = []
    i = 0
    while i < len(lines):
        ist = i
        [bc1Lines, i] = get1BcLines(lines, i)
        header = bc1Lines[0]
        dataLines = bc1Lines[1:]
        numLines = []
        nameLines = []
        grpNames = []
        for line in dataLines:
            words = deleteSp(line).split(",")
            try:
                a = int(words[0])
                numLines.append(line)
            except:
                nameLines.append(line)
                #grp名を取得
                if len(grpNames) == 0:
                    grpNames.append(words[0])
                else:
                    if grpNames[-1] != words[0]:
                        grpNames.append(words[0])
        #複数のgrp名があれば分割する
        if len(grpNames) > 1:
            #headerを含むblockを追加
            #  第1 block を作成
            bcLines = []
            bcLines += [header]
            for line in nameLines:
                if deleteSp(line).split(",")[0] == grpNames[0]:
                    bcLines += [line]
            bcLines += numLines
            #  第2 block 以降を作成
            for name in grpNames[1:]:
                bcLines += [header]
                for line in nameLines:
                    if deleteSp(line).split(",")[0] == name:
                        bcLines += [line]
            #newLinesに追加
            newLines += bcLines
        else:
            #そのままnewLinesに追加
            newLines += bc1Lines
    return newLines

#
#  convBoundaryConditions
#    境界条件名をFistr用に修正する
def convBoundaryConditions(lines):
    newLines = []
    #1文字目を修正
    for i in range(len(lines)):
        if lines[i][0] == "*":
            lines[i] = "!" + lines[i][1:]
    #内容を修正
    for i in range(len(lines)):
        if lines[i][0] == "!":
            words = deleteSp(lines[i]).split(",")
            words[0] = " ".join((lines[i].split(",")[0]).split()).upper()
            comms = convCont()
            for comm in comms:
                if words[0][1:] == comm[0]:
                    words[0] = "!" + comm[1]
            newLine = ", ".join(words) + "\n"
            newLines.append(newLine)
        else:
            newLines.append(lines[i])
    return newLines

#
#  convNoToName
#    node番号で指定しているものをgrpNameに修正する。
def convNoToName(lines):
    #直接nodeNoで設定しているnodeNoを取得
    newBcLines = []
    unDefNodes = []
    unDefElements = []
    bcItems = nItemsInBC()
    i = 0
    while i < len(lines):
        [bc1Lines, i] = get1BcLines(lines, i)
        if len(bc1Lines) > 1:
            header = bc1Lines[0]
            dataLines = bc1Lines[1:]
            [bcName, dummy] = getHeaderGrpName(bc1Lines)
            #node or elementを取得
            cName = ""
            for bcItem in bcItems:
                if bcItem[0] == bcName:
                    cName = bcItem[3]
                    break
            #数字をnode or elementに修正
            if cName != "":
                #dataLinesが10行以下の場合は、処理する
                if len(dataLines) <= 10:
                    #節点No、要素Noを各々のgroup名に変換する
                    for n in range(len(dataLines)):
                        dataLine = dataLines[n]
                        words = deleteSp(dataLine).split(",")
                        try:
                            a = int(words[0])
                            words[0] = cName + "_" + words[0]
                            dataLines[n] = ", ".join(words) + "\n"
                            if cName == "node":
                                unDefNodes.append(a)
                            elif cName == "element":
                                unDefElements.append(a)
                        except:
                            pass
                    newBcLines += [header] + dataLines
            else:
                newBcLines += bc1Lines
        else:
            newBcLines += bc1Lines
    #定義するnodeNoを取得
    nodes = []
    for node in unDefNodes:
        flag = 0
        for getNode in nodes:
            if node == getNode:
                flag = 1
                break
        if flag == 0:
            nodes.append(node)
    #nodeで追加する行を作成
    addLines = []
    if len(nodes) > 0:
        for node in nodes:
            line = "!NGROUP, NGRP=node_" + str(node) + "\n"
            addLines.append(line)
            line = str(node) + "\n"
            addLines.append(line)
    #定義するelementNoを取得
    elements = []
    for element in unDefElements:
        flag = 0
        for getEl in elements:
            if element == getEl:
                flag = 1
                break
        if flag == 0:
            elements.append(element)
    #elementで追加する行を作成
    if len(elements) > 0:
        for el in elements:
            line = "!EGROUP, EGRP=element_" + str(el) + "\n"
            addLines.append(line)
            line = str(el) + "\n"
            addLines.append(line)
    return [newBcLines, addLines]

#
#  combBoundaryConditions
#    変換した境界条件中で同じgroup名を上書きする
def combBoundaryConditions(lines):

    #  mergeLines
    #    両方のlinesを上書きする
    def mergeBoundaryLines(currLines, saveLines):
        header = currLines[0]
        currDataLines = currLines[1:]
        for currLine in currDataLines:
            currWords = deleteSp(currLine).split(",")
            curr = currWords[1:3]
            if len(curr) == 1:
                curr += [curr[0]]
            flag = 0
            saveDataLines = saveLines[1:]
            for i in range(len(saveDataLines)):
                saveLine = saveDataLines[i]
                saveWords = deleteSp(saveLine).split(",")
                save = saveWords[1:3]
                if len(save) == 1:
                    save += [save[0]]
                if curr == save:
                    saveDataLines[i] = ", ".join(currWords) + "\n"
                    flag = 1
                    break
            if flag == 0:
                saveDataLines += [currLine]
        ansLines = [header] + saveDataLines
        return ansLines

    #
    #  addNewBcLines
    #    同じgrp名の境界条件に追加して書き込み
    def addNewBcLines(bcLines, newLines):
        [bcName, grpName] = getHeaderGrpName(bcLines)
        if len(newLines) == 0:
            newLines += bcLines
        else:
            i = 0
            flag = 0
            while i < len(newLines):
                ist = i
                [newBcLines, i] = get1BcLines(newLines, i)
                if len(newBcLines) > 1:
                    [newBcName, newGrpName] = getHeaderGrpName(newBcLines)
                    if bcName == newBcName and grpName== newGrpName:
                        if bcName == "BOUNDARY":
                            bcLines = mergeBoundaryLines(bcLines, newBcLines)
                        newLines = newLines[:ist] + bcLines + newLines[i:]
                        flag = 1
                        break
            if flag == 0:
                newLines += bcLines
        return newLines

    newLines = []
    i = 0
    while i < len(lines):
        #1ヶの境界条件を取得
        [bcLines, i] = get1BcLines(lines, i)
        #同じgrp名中に境界条件を追加する
        newLines = addNewBcLines(bcLines, newLines)
    #各条件内のdataを整形する
    newLines = reformBcLines(newLines)
    return newLines

#
#  refineBoundaryConditions
#    境界条件を微修正
def refineBoundaryConditions(bcLines):
    i = 0
    while i < len(bcLines):
        ist = i
        [bc1Lines, i] = get1BcLines(bcLines, i)
        [bcName, grpName] = getHeaderGrpName(bc1Lines)
        bcName = bcName.upper()
        if bcName == "BOUNDARY":
            coord = deleteSp(bc1Lines[1]).split(",")[1:3]
            if coord == ["11", "11"]:
                #FIXTEMPに修正
                line = bcLines[ist]
                words = deleteSp(line).split(",")
                words[0] = "!FIXTEMP"
                bcLines[ist] = ", ".join(words) + "\n"
                line = bcLines[ist+1]
                words = deleteSp(line).split(",")
                bcLines[ist+1] = ", ".join([words[0], words[3]]) + "\n"
        elif bcName == "CFLUX":
            words = deleteSp(bcLines[ist+1]).split(",")
            if len(words) > 2:
                bcLines[ist+1] = ", ".join([words[0], words[2]]) + "\n"
        elif bcName == "DLOAD":
            line = bcLines[ist+1]
            words = deleteSp(line).split(",")
            if words[1].upper() == "CENTRIF" or words[1].upper == "centrif":
                words[1] = "CENT"
                bcLines[ist+1] = ", ".join(words) + "\n"
    return bcLines

#
#  seperateBcLines
#    cntとmsh側に分離する
def seperateBcLines(bcLines):
    mshLines = []
    cntLines = []
    i = 0
    while i < len(bcLines):
        [bc1Lines, i] = get1BcLines(bcLines, i)
        if len(bc1Lines) > 0:
            [header, grpName] = getHeaderGrpName(bc1Lines)
            if header == "INITIAL CONDITION":
                mshLines += bc1Lines
            else:
                cntLines += bc1Lines
    return [cntLines, mshLines]


#
#  reformBcLines
#    各境界条件を整形する
def reformBcLines(lines):

    #  getKindBc
    #    境界条件の種類を取得
    def getKindBc(lines):
        kindBc = []
        i = 0
        while i < len(lines):
            [bc1Lines, i] = get1BcLines(lines, i)
            if len(bc1Lines) != 0:
                [header, grpName] = getHeaderGrpName(bc1Lines)
                flag = 0
                for bcName in kindBc:
                    if bcName[:2] == [header, grpName]:
                        flag = 1
                        break
                if flag == 0:
                    kindBc.append([header, grpName, bc1Lines[0], []])
        return kindBc

    #  addBcData
    #    同じ境界条件で同じgroup名内に追加
    def addBcData(dataLines, bcName):
        #組数と初期値を取得
        bcItems = nItemsInBC()
        for bcItem in bcItems:
            if bcItem[0] == header.upper():
                nItems = bcItem[1]
                defValue = bcItem[2]
                break
        #配列を準備
        data = []
        for n in range(nItems):
            data.append([])
        #dataを読み込み（配列内に上書き）
        for line in dataLines:
            words = deleteSp(line).split(",")
            if words != [""]:
                words[0] = " ".join(line.split(",")[0].split())
                for n in range(nItems):
                    if len(data[n]) == 0 or data[n] == "\n":
                        data[n] = words
                        break
                    else:
                        if data[n][:2] == words[:2]:
                            data[n] = words
                            break
        #dataを整形
        newData = []
        for n in range(nItems):
            if len(data[n]) != 0:
                newData.append(", ".join(data[n]) + "\n")
        bcName[3] += newData
        return bcName

    #  checkReduse
    #    省略形を元に戻す
    def checkReduse(kindBc):
        for bcName in kindBc:
            if bcName[0] == "BOUNDARY":
                #初期値を取得
                bcItems = nItemsInBC()
                for bcItem in bcItems:
                    if bcItem[0] == "BOUNDARY":
                        defValue = bcItem[2]
                        break
                data = bcName[3]
                for i in range(len(data)):
                    line = data[i]
                    words = deleteSp(line).split(",")
                    if len(words) == 2:
                        words += [words[1]] + [str(defValue)]
                    elif len(words) == 3:
                        if words[2] == "":
                            words[2] = words[1]
                        words += [str(defValue)]
                    else:
                        if words[2] == "":
                            words[2] = words[1]
                        if words[3] == "":
                            words[3] = str(defValue)
                    data[i] = ", ".join(words) + "\n"
        return kindBc

    #  reorderBc
    #    data部は、group名が最初でnode（element）番号は、後
    def reorderBc(kindBc):
        for bcName in kindBc:
            data = bcName[3]
            num = []
            name = []
            for line in data:
                words = deleteSp(line).split(",")
                try:
                    a=int(words[0])
                    num.append(line)
                except:
                    name.append(line)
            bcName[3] = name + num
        return kindBc

    #境界条件の種類を取得
    kindBc = getKindBc(lines)
    newLines = []
    for bcName in kindBc:
        header = bcName[0]
        grpName = bcName[1]
        i = 0
        while i < len(lines):
            [bc1Lines, i] = get1BcLines(lines, i)
            if len(bc1Lines) != 0:
                [dHeader, dGrpName] = getHeaderGrpName(bc1Lines)
                if header == dHeader and grpName == dGrpName:
                    dataLines = bc1Lines[1:]
                    bcName = addBcData(dataLines, bcName)
    #省略形を修正
    kindBc = checkReduse(kindBc)
    #記述順を修正（nodeGrp名が最初）
    kindBc = reorderBc(kindBc)
    #data行を作成
    for bcName in kindBc:
        line = bcName[2]
        newLines.append(line)
        newLines += bcName[3]
    return newLines

#
#  getHeaderGrpName
#    headerとgrpNameを取得
def getHeaderGrpName(lines):
    ans = []
    if len(lines) >= 2:
        #header = deleteSp(lines[0]).split(",")[0][1:]
        header = " ".join(lines[0].split(",")[0][1:].split())
        grpName = ""
        for line in lines[1:]:
            words = deleteSp(line).split(",")
            try:
                a = int(words[0])
            except:
                grpName = words[0]
                break
        ans = [header, grpName]
    return ans

#
#  getHeaderAllGrpNames
#    header内の全てのgrpNameを取得
def getHeaderAllGrpNames(lines):
    ans = []
    if len(lines) >= 2:
        header = deleteSp(lines[0]).split(",")[0][1:]
        names = []
        for line in lines[1:]:
            words = deleteSp(line).split(",")
            try:
                a = int(words[0])
            except:
                names.append(words[0])
        if len(names) >= 1:
            names.sort()
            grpNames = []
            for name in names:
                flag = 0
                for grpName in grpNames:
                    if name == grpName:
                        flag == 1
                        break
                if flag == 0:
                    grpNames.append(name)
        ans = [header, grpNames]
    return ans        

#
#  deleteSameBcLines
#    同じ境界条件、group名部を削除する
def deleteSameBcLines(cntLines, bcLines):
    i = 0
    while i < len(bcLines):
        [bc1Lines, i] = get1BcLines(bcLines, i)
        header = deleteSp(bc1Lines[0]).split(",")[0]
        grpName = deleteSp(bc1Lines[1]).split(",")[0]
        cntLines = delete1Bc(cntLines, header, grpName)
    #cntLines = topHeader + cntLines
    return cntLines

#
#  addBcLines
#    境界条件をcntファイルに追加
def addBcLines(cntLines, bcLines):
    #挿入場所を取得
    i = getInsertPoint(cntLines, "!BOUNDARY", ["Boundary", "BOUNDARY", "boundary"])
    newLines = cntLines[:i] + bcLines + cntLines[i:]
    return newLines

#
#  writeMeshFile
#    mshファイルの最後にaddLinesを追加する
def writeMeshFile(fileName, addLines):
    if len(addLines) == 0:
        return

    f=open(fileName); meshLines=f.readlines(); f.close()
    n = len(meshLines) - 1
    ist = 0
    while n >= 0:
        line = meshLines[n]
        if line[:len("!END")] == "!END":
            ist = n
            break
        n -= 1
    meshLines = meshLines[:ist] + addLines + meshLines[ist:]
    f=open(fileName, "w")
    for meshLine in meshLines:
        f.write(meshLine)
    f.close()


#  delete1Bc
#    該当する境界条件を削除
def delete1Bc(cntLines, header, grpName):
    i = skipHeader(cntLines, 0)
    newLines = cntLines[:i]
    while i < len(cntLines):
        [cont, i] = get1block(cntLines, i)
        if len(cont) >= 2:
            cntHeader = deleteSp(cont[0]).split(",")[0]
            cntGrpName = deleteSp(cont[1]).split(",")[0]
            if (cntHeader == header and cntGrpName == grpName):
                for line in cont:
                    if line[0] == "#":
                        newLines += [line]
                pass
            else:
                newLines += cont
        else:
            newLines += cont
    return newLines

#  get1BcLines
#    1ヶの境界条件を取得
def get1BcLines(lines, i):
    #ie = i
    ie = len(lines)
    bcLines = []
    flag = 0
    while i < len(lines):
        line = lines[i]
        if ( (line[0] == "!" or line[0] == "*") and
             (lines[i+1][0] != "!" and lines[i+1][0] != "*") ):
            #BCを取得
            bcLines.append(line)
            i += 1
            while i < len(lines):
                if lines[i][0] != "!" and lines[i][0] != "*":
                    bcLines.append(lines[i])
                    flag = 1
                else:
                    break
                i += 1
        if flag == 1:
            ie = i
            break
        i += 1
    return [bcLines, ie]

#  get1block
#    1ヶのコマンド行を取得
def get1block(lines, i):
    sp = skipHeader(lines, i)
    if sp < len(lines):
        ep = skipHeader(lines, sp+1)
        newLines = lines[sp:ep]
    else:
        newLines = []
        ep = len(lines)
    return [newLines, ep]

#  skipEndHeader
#    "!" or "#"文字を検索
def skipEndHeader(lines, i):
    ip = len(lines)
    while i < len(lines):
        if lines[i][0] == "!" or lines[i][0] == "#":
            ip = i
            break
        i += 1
    return ip
    

#  skipHeader
#    "!"文字を検索
def skipHeader(lines, i):
    ip = len(lines)
    while i < len(lines):
        if lines[i][0] == "!":
            ip = i
            break
        i += 1
    return ip

#  getInsertPoint
#    挿入場所を取得
#    keywordsの次の行No、存在しない場合はheaderの行Noが戻る
def getInsertPoint(lines, header, keywords):
    ip = 0
    #keywordを検索
    i = 0
    while i < len(lines):
        line = lines[i]
        if line[0] == "#":
            flag = 0
            for key in keywords:
                if line.find(key) >= 0:
                    flag = 1
                    break
            if flag == 1:
                i += 1
                if lines[i][0] == "#":
                    i += 1
                ip =i
                break
        i += 1
    #keywordに該当したか？
    if ip == 0:
        #headerを検索
        i = 0
        while i < len(lines):
            line = lines[i]
            if line[:len(header)] == header:
                ip = i
                break
            i += 1
    return ip

#
#  deleteComments
#    コメント行を削除
def deleteComments(lines):
    newLines = []
    for line in lines:
        if line[:2] != "**":
            if line.replace(" ", "").replace("\t", "") != "\n":
                newLines.append(line)
    return newLines
 
#
#  readFile
#    ファイル読み込み
def readFile(fileName):
    lines = []
    f=open(fileName)
    for line in f.readlines():
        if line != "\n" and line != "\r\n":
            #空白行を削除
            if line[-2:] == "\r\n":
                #行末を統一
                line = line[:-2] + "\n"
            lines.append(line)
    f.close()
    return lines

#
#  writeFile
#    ファイルに書き込み
def writeFile(fileName, lines):
    fw = open(fileName, "w")
    for line in lines:
        fw.write(line)
    fw.close()

#
#  abaqusinp2fistrcnt
#  ------------------
#    inpファイルから境界条件を読み取りcntファイルに書き込む
def abaqusinp2fistrcnt(fileDir, abaName):
    print()
    print ("boundary conditions in inp file will be convert to FrontISTR. ")
    #inpファイルの読み込み
    fileName = fileDir + os.sep + abaName + ".inp"
    lines = readFile(fileName)
    #コメント削除
    lines = deleteComments(lines)
    #境界条件を取得
    print ("getting to convrt items...")
    bcLines = getBoundaryConditions(lines)
    #境界条件を修正（順番に該当分を全て取得）
    print ("converting above items...")
    bcLines = convBoundaryConditions(bcLines)
    #node番号をnodeGrp名に修正
    print ("changing node or element number to its group name...")
    [bcLines, addMeshLines] = convNoToName(bcLines)
    #境界条件とgrpNameをチェック（heder内には1ヶgrp名にする）
    bcLines = checkGrpNameInHeader(bcLines)
    #修正した境界条件をまとめる（同じgroup名があれば上書きする）
    print ("overwritiing to same keyword and same group name...")
    bcLines = combBoundaryConditions(bcLines)
    #境界条件をrefineする。
    print ("converting detail part...")
    bcLines = refineBoundaryConditions(bcLines)
    #cntとmsh側を分離する
    [bcLines, newAddMeshLines] = seperateBcLines(bcLines)
    addMeshLines += newAddMeshLines
    #cntファイル読み込み
    fileName = fileDir + os.sep + "FistrModel.cnt"
    cntLines = readFile(fileName)
    #追加部分を削除（有れば）
    cntLines = deleteSameBcLines(cntLines, bcLines)
    #cntファイルに境界条件を追加
    newLines = addBcLines(cntLines, bcLines)
    #cntファイル書き込み
    print ("writing cnt file...")
    writeFile(fileName, newLines)
    #mshファイルに書き込み
    print ("writing msh file...")
    writeMeshFile(fileDir+os.sep+"FistrModel.msh", addMeshLines)
    print ("above items have be converted.")



if __name__=='__main__':
    abaNameDir = sys.argv[1]
    if abaNameDir[:len("-h")] == "-h" or abaNameDir[:len("--help")] == "--help":
        printMsg()
    else:
        fileDir = os.sep.join(abaNameDir.split(os.sep)[:-1])
        if fileDir == "":
            fileDir = os.getcwd()
        abaName = abaNameDir.split(os.sep)[-1]
        if abaName.split(".")[-1] == "inp":
            name = abaName[:-4]
        else:
            name = abaName
        abaqusinp2fistrcnt(fileDir, name)

